import logging
import grpc
from zeebe_grpc import gateway_pb2, gateway_pb2_grpc


class ZeebeWorker:
    """Represents a zeebe Worker"""

    def __init__(self, name, brocker, timeout=1000, max_jobs=1, pollingtime=5):
        self.name = name
        self.brocker = brocker
        self.timeout = timeout  # 1000 ms = 1 s
        self.max_jobs = max_jobs
        self.pollingtime = pollingtime  # 5s
        self.tasks = dict()

    def register_task(self, task_type, function, custom_timeout=None):
        self.tasks[task_type] = {
            "function": function,
            "timeout": custom_timeout or self.timeout,
        }

    def run(self):
        with grpc.insecure_channel(self.brocker) as channel:
            stub = gateway_pb2_grpc.GatewayStub(channel)

            while True:
                logging.info("Polling Jobs...")

                for task_type, task in self.tasks.items():
                    function = task.get("function")
                    timeout = task.get("timeout")

                    logging.info(f"Activating Job Type {task_type}")

                    activate_jobs_response = stub.ActivateJobs(
                        gateway_pb2.ActivateJobsRequest(
                            type=task_type,
                            timeout=timeout,
                            worker=self.name,
                            maxJobsToActivate=self.max_jobs,
                        )
                    )

                    for response in activate_jobs_response:
                        for job in response.jobs:
                            try:
                                function(job)
                                stub.CompleteJob(
                                    gateway_pb2.CompleteJobRequest(
                                        jobKey=job.key,
                                    )
                                )
                                logging.info("Job Completed")
                            except Exception as e:
                                stub.FailJob(gateway_pb2.CompleteJobRequest(jobKey=job.key))
                                logging.error(f"Job Failed {e}")
